@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.102 2006/01/17 02:16:11 tkaroonb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.104 2006/03/27 21:06:21 richb Exp $
  *
  *  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -375,7 +375,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     '-',
     M_NONE,
     do_calc,
-    NULL,
+    N_("-"),
     unop | binop
 },
 {
@@ -400,7 +400,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     's',
     M_NONE,
     do_immed, 
-    NULL,
+    N_("Sqrt"),
     func
 },
 {
@@ -1147,7 +1147,7 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
 void
 do_calctool(int argc, char **argv)
 {
-    char *ptr, title[MAXLINE];
+    char *ptr;
     int i;
 
     init_buttons();            /* Setup the buttons for "Basic" mode. */
@@ -1210,12 +1210,11 @@ do_calctool(int argc, char **argv)
         }
     }
 
-    SPRINTF(title, "%s - %s", v->tool_label, _(mstrs[(int) v->modetype]));
-    set_title(FCP_KEY, title);
+    set_main_title(v->modetype);
 
     show_display(v->MPdisp_val);     /* Output in correct display mode. */
 
-    memset(&(v->e), 0, sizeof(struct exprm_state)); /* clear expression mode state */
+    memset(&(v->h), 0, sizeof(struct exprm_state_history)); /* clear expression mode state history*/
 
     start_tool();                    /* Display the calculator. */
 }
