@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.h,v 1.5 2004/08/01 08:25:30 sampie Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.h,v 1.8 2006/01/09 16:30:20 richb Exp $
  *
- *  Copyright (C) 2004 Sami Pietila
+ *  Copyright (C) 2004-2006 Sami Pietila
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,12 +25,13 @@
 #include "calctool.h"
 #include "extern.h"
 
-int do_trigfunc(int s[MP_SIZE], int t[MP_SIZE]);
-
 void do_factorial(int *, int *);
-void do_immedfunc(int s[MP_SIZE], int t[MP_SIZE]);
-void do_portionfunc(int num[MP_SIZE]);
-
 void exp_append(char *text);
 
+struct exprm_state *get_state(void);
+void new_state(void);
+void perform_undo(void);
+void perform_redo(void);
+void clear_undo_history(void);
+
 #endif /*FUNCTIONS_H*/
